# Prepare Track 1 for DDP export
# Moves items on track 1 to conform with DDP export. First item will start at 2 seconds, each subsequent item will start with a 2 seconds intervall to the previous, quantized to 75 frames/second.
# Places markers with the suffix "#" and the active take name at each item start
# Places Markers with the suffix "!" at project start and each item end (end markers quantized to the next frame)
# Previously existing markers will be deleted (might add prompt to save as SWS marker set).

RPR_Undo_BeginBlock()

#Remove all markers:
Marker = RPR_EnumProjectMarkers2(0, 0, 0, 0, 0, 0, 0)
IDX = Marker[0]
while IDX > 0:
	MarkIdx = Marker[7]  # gets the last (seventh) item of the return array, the marker index number
	RPR_DeleteProjectMarker( 0, MarkIdx, 0)
	Marker = RPR_EnumProjectMarkers2(0, 0, 0, 0, 0, 0, 0)
	IDX = Marker[0]

RPR_Main_OnCommand(40898, 0) # Renumbers marker ids in order (of course there are none at this point) Was the only workaround I know so far to get the undo behave nice.. If I delete this line, Undo has to be hit twice to get to the previous state.

RPR_Main_OnCommand(41238, 0) # save selection set 10
RPR_Main_OnCommand(40289, 0) # unselects all items

# unselect all tracks, select the first track.	Just for convenience. Not necessary to make the script work:
TrackCount = RPR_CountTracks(0)
TrackIndex = 0
while TrackIndex < TrackCount:
	track = RPR_GetTrack(0, TrackIndex)
	RPR_SetTrackSelected(track, 0)
	TrackIndex += 1
	
TrackOne = RPR_GetTrack(0, 0) #The left 0 is "current project" (not clear on this), the right 0 means "the track with the lowest track number (track 1)"
RPR_SetTrackSelected(TrackOne, 1)

ItemCount = RPR_CountTrackMediaItems(TrackOne)
StartOfNext = 2 # first item shall start on 2 seconds in.
ItemIndex = 0
ItemNumber = 0
CurrentItem = 0
Message = 0

RPR_AddProjectMarker(0, 0, 0, 8, "!", 0) #add a "!" marker at zero position

item = RPR_GetMediaItem(0, ItemIndex)
RPR_SetMediaItemInfo_Value(item, "B_UISEL", 1)   # "physically" selects the first of the items.

while ItemIndex < ItemCount:
	while ItemNumber < ItemIndex:  # this small inner loop does the "select 1st, 2nd, 3rd ... part. With each iteration of the outer loop, this loop will run through one more time - resulting in one item further right to be the selected one.
		RPR_SetMediaItemInfo_Value(item, "B_UISEL", 0) #unselect the current item
		item = RPR_GetMediaItem(0, ItemNumber + 1)
		RPR_SetMediaItemInfo_Value(item, "B_UISEL", 1)  #select the next one
		ItemNumber += 1
		
	item = RPR_GetSelectedMediaItem(0, 0) # gets the first selected item, so we can ask it for it's properties
	item_start = RPR_GetMediaItemInfo_Value(item, "D_POSITION")	# getting the current item's start time to calculate nudge amount and direction.
	active_take = RPR_GetActiveTake(item) # pointer to active take
	take_name = RPR_GetSetMediaItemTakeInfo_String(active_take, "P_NAME", "", False)[3] # gets name of active take
	marker_name = "# " + take_name # prefixes the take's name with "#". This will be the name of our "#" markers
	
# select all following items, so they will get nudged all together in the "apply nudge" step:
	while CurrentItem + 1 < ItemCount:
		item_a = RPR_GetMediaItem(0, CurrentItem + 1)
		RPR_SetMediaItemInfo_Value(item_a, "B_UISEL", 1)	
		CurrentItem += 1	
	if StartOfNext < item_start:  # finds the relation of actual item start and the desired position
		RPR_ApplyNudge(0, 0, 0, 1, item_start - StartOfNext, 1, 0)	# nudges all selected items leftward, so the first selected item starts at the desired (StartOfNext) position.
	else:
		RPR_ApplyNudge(0, 0, 0, 1, StartOfNext - item_start, 0, 0)  # nudges all selected items rightward, so the first selected item starts at the desired (StartOfNext) position.
	RPR_AddProjectMarker(0, 0, StartOfNext, 8, marker_name, ItemNumber + 1) # puts "#" marker at item start.
	
	item_start_after = RPR_GetMediaItemInfo_Value(item, "D_POSITION")	# getting the current item's new start time to calculate endposition
	item_length = RPR_GetMediaItemInfo_Value(item, "D_LENGTH")	# getting the current item's lenght to calculate the placement of the next item (StartOfNext for the next loop iteration)
	item_endpos = item_start_after + item_length
	
	framediv = item_endpos / 0.01333333333333333333333
	FrameCount = int(framediv)  # finds the frame during which the item end position is
	item_endframe = (FrameCount + 1) * 0.01333333333333333333333 # quantizes the position to the next CD 75 frame/sec grid
	item_length_frame = item_endframe - item_start_after
	
	if item_length_frame < 4:
		Message += 1	
		tooShort = round(4 - item_length, 2)
		item_length_round = round (item_length, 2)
		msgText = "Item #" + str(ItemNumber + 1) + " (" + str(take_name) + ")  is ~ " + str(item_length_round) + "sec. That's " + str(tooShort) + " sec too short for CD. \n"
		if Message == 1:
			RPR_ShowConsoleMsg("")
		RPR_ShowConsoleMsg(msgText)
	RPR_AddProjectMarker(0, 0, StartOfNext + item_length_frame, 8, "!", 0)  # puts "!" marker at item end	
	StartOfNext = StartOfNext + item_length_frame +2  # prepares desired item start for the next loop iteration.
	ItemIndex +=1 
if item_endframe > 4440:
	Message += 1
	tooLong = round(item_endframe - 4440, 2)
	tooLong_min = tooLong / 60
	min_round = int(tooLong_min)
	sec = tooLong - min_round*60
	if Message == 1:
		RPR_ShowConsoleMsg("")		
		msgLong ="Project is ~ " + str(min_round) + "min, " + str(sec) + "sec " + "(" + str(tooLong) + " seconds) longer than 74 minutes!  \n"
	else:
		msgLong ="\nProject is ~ " + str(min_round) + "min, " + str(sec) + "sec " + "(" + str(tooLong) + " seconds) longer than 74 minutes!  \n"

	RPR_ShowConsoleMsg(msgLong)
if Message > 0:
	RPR_ShowConsoleMsg("\n")
if Message == 0:
	RPR_ShowConsoleMsg("")
	RPR_ShowConsoleMsg("alright")
RPR_UpdateTimeline()
RPR_Main_OnCommand(41248, 0) # load selection set 10

RPR_Undo_EndBlock("Prepare item positions and markers for DDP",0)

#---